/***************************************************************************************
屋外に設置した子機XBee ZB搭載ガイガーカウンタ専用の親機です。
※DF ROBOT製の液晶がSDカードの信号と干渉するのでAdafruit製の液晶を使用しています。

                                                       Copyright (c) 2013 Wataru KUNINO
***************************************************************************************/
#include <xbee.h>
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
#include <SD.h>
#include "pitches.h"

#define PIN_SDCS    4                               // SDのチップセレクトCSポート番号
#define PIN_BUZZER  2                               // Digital 2にスピーカを接続

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();
File file;                                          // SDファイルの定義

unsigned int min20;                                 // 次回の測定期限[20分単位]
byte err;                                           // エラーのカウント

void setup() {
    lcd.begin(16, 2);                               // LCDのサイズを16文字×2桁に設定
    lcd.print("Example 63 LOG  ");                  // 文字を表示
    while(SD.begin(PIN_SDCS)==false){               // SDカードの開始
        delay(5000);                                // 失敗時は5秒ごとに繰り返し実施
    }
    xbee_init( 0 );                                 // XBee用COMポートの初期化
    xbee_at("ATSP0AF0");                            // 最大スリープ時間を本機に設定
    min20 = 0;                                      // 次回の測定期限
}

void loop() {
    byte i,data;
    float value;                                    // 受信データの代入用
    XBEE_RESULT xbee_result;                        // 受信データ(詳細)
    unsigned long time=millis()/1000;               // 時間[秒]の取得(約50日まで)

    /* 子機ガイガーから20分～40分以上の受信が無ければペアリングを行う */
    unsigned int time20 = (unsigned int)(time/60/20);
    if( time20 >= min20 && time20 < 3579){          // 約20～40分の経過を確認
        lcd.setCursor(0,1);
        lcd.print("Waiting 4 Geiger");
        if( xbee_atnj(30)==0 ) err++;               // 30秒のジョイン許可
        lcd.clear();
        min20 = time20 + 2;                         // 約20～40分後の時間[20分単位]
        if( min20 >= 3579 ) min20 = 1;              // 約50日後にmillisが0になる対策
    }

    data=xbee_rx_call( &xbee_result );              // データを受信
    if( xbee_result.MODE == 0x00 ) return;          // データが無い時は終了
    
    if( xbee_result.MODE == MODE_UART ){            // UARTシリアルデータ
        min20 = time20 + 2;                         // 約20～40分後の時間[20分単位]
        if( min20 >= 3579 ) min20 = 1;              // 約50日後にmillisが0になる対策
        i=0;                                        // エラー検出用フラグとして使用
        if( data == 23 ) for(i=0; i<22; i++){       // データ長=23
            if(!isdigit(xbee_result.DATA[i]) && xbee_result.DATA[i] != (byte)',') break;
        }
        if( i==22 && xbee_result.DATA[22]=='\r'){
            /*
            受信フォーマット        →  表示フォーマット
            HHMMSS,Cnt,uSv,Vlt,Tmp/     0.10uSv/h  25.0C
                                        HH:MM:SS 0 4.20V    */
            lcd.clear();
            lcd.print( (char)xbee_result.DATA[11] );
            lcd.print( '.' );
            lcd.print( (char)xbee_result.DATA[12] );
            lcd.print( (char)xbee_result.DATA[13] );
            lcd.print( " uSv/h" );
            lcd.setCursor(11,0);
            lcd.print( (char)xbee_result.DATA[19] );
            lcd.print( (char)xbee_result.DATA[20] );
            lcd.print( '.' );
            lcd.print( (char)xbee_result.DATA[21] );
            lcd.print( 'C' );
            lcd.setCursor(0,1);
            lcd.print( (char)xbee_result.DATA[0] );
            lcd.print( (char)xbee_result.DATA[1] );
            lcd.print( ':' );
            lcd.print( (char)xbee_result.DATA[2] );
            lcd.print( (char)xbee_result.DATA[3] );
            lcd.print( ':' );
            lcd.print( (char)xbee_result.DATA[4] );
            lcd.print( (char)xbee_result.DATA[5] );
            lcd.setCursor(11,1);
            lcd.print( (char)xbee_result.DATA[15] );
            lcd.print( '.' );
            lcd.print( (char)xbee_result.DATA[16] );
            lcd.print( (char)xbee_result.DATA[17] );
            lcd.print( 'V' );
            
            file=SD.open("GEIGER.CSV",FILE_WRITE);      // 書込みファイルのオープン
            if(file == true){                               // オープンが成功した場合
                for(i=0;i<22;i++)
                    file.print((char)xbee_result.DATA[i]);  // 受信した文字列をSDへ保存
                file.println( "" );                         // SDへ改行を保存
                file.close();                               // ファイルクローズ
            }else{
                lcd.setCursor(11,1);
                lcd.print( "SD-ER" );
            }
            if( xbee_result.DATA[11] > (byte)'0' ){     // 1uSv/h以上の時
                tone(PIN_BUZZER,NOTE_A7,100);           // ブザーに音階A7を出力
                lcd.setCursor(11,1);
                lcd.print( "HIGH!" );
            }
        }else err++;
        lcd.setCursor(9,1);
        lcd.print( err , HEX );
    }
}
