/*********************************************************************
本ソースリストおよびソフトウェアは、ライセンスフリーです。
利用、編集、再配布等が自由に行えますが、著作権表示の改変は禁止します。

なお、Radiation-Watch.orgのソースコードを含みます。

                               Copyright (c) 2012-2013 Wataru KUNINO
                               http://www.geocities.jp/bokunimowakaru/
*********************************************************************/

//////////////////////////////////////////////////
// Radiation-Watch.org
// URL http://www.radiation-watch.org/
//////////////////////////////////////////////////
/*

Radiation-watch.orgプロジェクトは開発成果をクリエイティブ・コモンズ(CC 
BY 3.0)ライセンスの元でオープンソース化しています。以下では、ポケット
ガイガーの回路図、仕様、ソースコード、実験結果を、順次公開します。
これらは全世界において、商用・非商用を問わず自由に再配布・改変・公開す
ることができます。なお一部のコンテンツは英語またはオランダ語のみとなり
ます。

POCKET GEIGER by Radiation-Watch.org is licensed under a Creative 
Commons Attribution 3.0 Unported License.

*/

/*
使い方：
	(1) ライブラリのダウンロード
	　　当方のサイトから「PC/H8/Arduino用 管理lib. ZB Coordinator API 」を
	　　ダウンロードします。ファイル名＝xbeeCoord_XXX.zip (XXXは数字)
	　　　　http://www.geocities.jp/bokunimowakaru/diy/xbee/xbee-download.html
	(2) ライブラリのインストール
	　　ZIPファイルの中の「xbee_arduino」フォルダ内の「XBee」ファルダを	
	　　 arduinoをインストールしたフォルダ(例 C:\arduino)の中の「libraries」
	　　 フォルダにインポートします。
	　　　　C:\arduino\libraries\XBee\xbee.h
	　　　　C:\arduino\libraries\XBee\xbee.cpp
	　　　　C:\arduino\libraries\XBee\examples\...
	(3) Arduino IDE を起動します。(既に起動していた場合、一度、終了して再起動)
	(4) ソースにXBeeモジュールのIEEEアドレスを追加
	　　本ソースの「byte dev_gpio」の宣言部分に記載しているZigBee IEEEアドレス
	　　を、お手持ちの子機のアドレス(XBeeモジュールの裏面に記載)に変更します。
	(6) Arduinoマイコンボードに書き込みます。ATmega 328で動作確認しています。
*/

/*********************************************************************
インポート処理						※ライブラリ XBeeに必要な処理です
*********************************************************************/

#include <xbee.h>
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/power.h>
#include <Wire.h>
#include <DS3231.h>

/*********************************************************************
アプリケーション					※以下を編集してアプリを作成します
*********************************************************************/

#define		PIN_LED				13			// LED(SDのCLKと共通)
#define		PIN_GEIGER			14			// A0 ポケガの電源
#define		PIN_PULSE			15			// A1 ポケガ放射線パルス信号
#define		PIN_NOISE			16			// A2 ポケガノイズ検出信号
#define		PIN_XB_SLEEP		17			// A3 XBeeのスリープ端子
#define		PIN_BATTERY			7			// バッテリ電圧入力(アナログ)

#define		SLEEP_DURATION		900			// 15分 休止間隔(秒)
#define		TIME_MEAS			180			// 3分 測定時間(秒)

//The following code is taken from sleep.h as Arduino Software v22 (avrgcc) in w32 does not have the latest sleep.h file
#define sleep_bod_disable() \
{ \
  uint8_t tempreg; \
  __asm__ __volatile__("in %[tempreg], %[mcucr]" "\n\t" \
                       "ori %[tempreg], %[bods_bodse]" "\n\t" \
                       "out %[mcucr], %[tempreg]" "\n\t" \
                       "andi %[tempreg], %[not_bodse]" "\n\t" \
                       "out %[mcucr], %[tempreg]" \
                       : [tempreg] "=&d" (tempreg) \
                       : [mcucr] "I" _SFR_IO_ADDR(MCUCR), \
                         [bods_bodse] "i" (_BV(BODS) | _BV(BODSE)), \
                         [not_bodse] "i" (~_BV(BODSE))); \
}

DS3231 RTC;		//Create RTC object for DS3231 RTC come Temp Sensor 
static DateTime interruptTime;

float cpm = 0;					// 現在のカウント数
int cpmTimeSec=0;				// cpm計算用の時間[sec]
byte port=0;
byte dev[] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00};	// オール0はCoordinator

void xbee_wake(){
	digitalWrite(PIN_XB_SLEEP, HIGH);		// XBeeのスリープ設定
	delay(10);
	digitalWrite(PIN_XB_SLEEP, LOW);		// XBeeのスリープ解除
	delay(10);
}

void setup(){
	/*Initialize INT0 pin for accepting interrupts */
	PORTD |= 0x04; 
	DDRD &=~ 0x04;
	pinMode(4, INPUT);						// extern power
	pinMode(PIN_GEIGER, OUTPUT);			// ポケガの電源
	pinMode(PIN_LED, OUTPUT);				// LED(SDのCLKと共通)
	pinMode(PIN_XB_SLEEP, OUTPUT);			// XBeeのスリープ端子
	pinMode(PIN_PULSE, INPUT_PULLUP);		// ポケガ放射線パルス信号
	pinMode(PIN_NOISE, INPUT_PULLUP);		// ポケガノイズ検出信号
	analogReference(INTERNAL);				// ADCに内部の基準電圧を使用する
	
	Wire.begin();
	RTC.begin();

	xbee_wake();							// XBee起動
	xbee_init( 0x00 );						// XBee用COMポートの初期化
	xbee_atcb(4);							// ネットワーク初期化
	while( xbee_ping(dev) == 0xFF ){		// ping応答を確認
		while( xbee_atai() > 0x01 ){		// ネットワーク参加状況を確認
			for(int i=0;i<60;i++){
				digitalWrite(PIN_LED,i%2);	// LEDの高速点滅 60x(30+20)ms = 3秒
				xbee_wake();				// XBee起動
				delay(30);
			}
			xbee_atcb(1);					// ネットワーク参加ボタンを押下
		}
	}

	xbee_from(dev);							// ping応答のあったアドレスを保存
	xbee_wake();							// XBee起動
	xbee_at("ATSP07D0");					// 受信間隔を20秒に設定
	xbee_at("ATSM05");						// Cyclic Sleep + PIN wakeモードに設定

	digitalWrite(PIN_GEIGER, LOW);			// ポケガの電源をオン
	
	//Enable Interrupt 初回
	attachInterrupt(0, INT0_ISR, LOW);		// Only LOW level interrupt can wake up from PWR_DOWN
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	DateTime  start = RTC.now();
	interruptTime = DateTime( start.get() + SLEEP_DURATION );
}

int i;
boolean pulse;					// 放射線パルス信号の論理
boolean pulse_prev =HIGH;		// 放射線パルス信号の前回値
boolean disp_batt = LOW;		// バッテリ残量表示

DateTime now;					// 現在時刻
char s[40];						// 送信用の文字列
char sb[10];					// 文字列生成用のバッファ
int8_t time_prev;				// 前回時刻の秒
byte buttons = 0;

float temp;						// 温度
float battery = 0;				// 電池電圧

int signCount=0;				// 信号のカウント回数
int noiseCount=0;				// ノイズのカウント回数
float cpmBuff;					// 出力演算用
float uSvBuff;
float min;
const float alpha=53.032;		// cpm = uSv × alpha

void loop(){
	
	/* 測定 */
	for( i=0 ; i<10000 ; i++){
		pulse = digitalRead( PIN_PULSE );
		if( pulse_prev != pulse ){
			if( pulse == LOW )signCount++;			//放射線パルス(Low)をカウント
			pulse_prev = pulse;
		}
		if( digitalRead( PIN_NOISE ) == HIGH ){
			noiseCount++;							//ノイズ(High)をカウント
		}
	}
	digitalWrite(PIN_LED, !(digitalRead( PIN_LED )));	// LED点滅

	now = RTC.now();								// get the current date-time
	
	if( ( now.second() % 6 == 0 ) && (time_prev != now.second()  ) ){	// 6秒ごとの処理
		time_prev = now.second() ;
		
		RTC.convertTemperature();					// convert current temperature into registers
		temp = RTC.getTemperature();				// Read temperature sensor value
		battery = ((float)analogRead(PIN_BATTERY)-500)/160+3.2;
		
		if(noiseCount == 0){						// ノイズが検出されないとき
			cpm += (float)signCount;				// カウント回数の蓄積
			cpmTimeSec += 6 ;						// cpmを求めるときに使用する計測時間
		}
		min = (float)cpmTimeSec / 60.0;				// 現在の計測時間(最大TIME_MEAS分)
		if(min != 0.){								//cpm、uSv/h、uSv/hの誤差をそれぞれ計算する
			cpmBuff = cpm / min;
			uSvBuff = cpm / min / alpha;
		}else{										//0割り算のときは、0にする
			cpmBuff = 0;
			uSvBuff = 0;
		}

		//次のループのための変数の初期化
		signCount=0;
		noiseCount=0;
		
		//測定終了処理
		if( cpmTimeSec > TIME_MEAS ){
			sprintf(s, "%02d%02d%02d", now.hour(), now.minute(), now.second() );
			if( cpm < 1000 ) sprintf( sb ,",%03d", (int)cpm );
			else sprintf( sb ,",XXX" );
			strcat( s, sb );
			if( uSvBuff < 10 ) sprintf( sb ,",%03d", (int)(uSvBuff*100) );
			else sprintf( sb ,",XXX" );
			strcat( s, sb );
			sprintf( sb ,",%03d", (int)(battery*100) );
			strcat( s, sb );
			sprintf( sb ,",%03d", (int)(temp*10) );
			strcat( s, sb );
			xbee_wake();							// XBee起動
			xbee_address( dev );					// 文字列の送信アドレス設定
			xbee_putstr( s );						// 文字列の送信
			delay(100);								// 送信完了待ち
			digitalWrite(PIN_LED, LOW);				// LED消灯
			digitalWrite(PIN_XB_SLEEP, HIGH);		// XBeeのスリープ設定
			digitalWrite(PIN_GEIGER, HIGH);			// ポケガの電源をOFF
			//Enable Interrupt 
			RTC.clearINTStatus(); //This function call is  a must to bring /INT pin HIGH after an interrupt.
			RTC.enableInterrupts(interruptTime.hour(),interruptTime.minute(),interruptTime.second());    // set the interrupt at (h,m,s)
			attachInterrupt(0, INT0_ISR, LOW);  //Enable INT0 interrupt (as ISR disables interrupt). This strategy is required to handle LEVEL triggered interrupt
			//Power Down routines
			cli(); 
			sleep_enable();			// Set sleep enable bit
			sleep_bod_disable();	// Disable brown out detection during sleep. Saves more power
			sei();
			delay(10);
			//Shut down all peripherals like ADC before sleep. Refer Atmega328 manual
			power_all_disable();	//This shuts down ADC, TWI, SPI, Timers and USART
			sleep_cpu();			// Sleep the CPU as per the mode set earlier(power down)  
			sleep_disable();		// Wakes up sleep and clears enable bit. Before this ISR would have executed
			power_all_enable();		//This shuts enables ADC, TWI, SPI, Timers and USART
			delay(10);				//This delay is required to allow CPU to stabilize
			digitalWrite(PIN_GEIGER, LOW);			// ポケガの電源をON
			delay(1000);
			cpm = 0;
			cpmTimeSec = 0;
		}
	}
}

//Interrupt service routine for external interrupt on INT0 pin conntected to DS3231 /INT
void INT0_ISR(){
	//Keep this as short as possible. Possibly avoid using function calls
	detachInterrupt(0); 
	interruptTime = DateTime(interruptTime.get() + SLEEP_DURATION);  //decide the time for next interrupt, configure next interrupt  
}

