// #include <Wire.h>
// #include <DS3231.h>
// DS3231 RTC;		//Create RTC object for DS3231 RTC come Temp Sensor 
// #define		SLEEP_DURATION		900			// 15分 休止間隔(秒)

#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/power.h>

#define sleep_bod_disable() \
{ \
  uint8_t tempreg; \
  __asm__ __volatile__("in %[tempreg], %[mcucr]" "\n\t" \
                       "ori %[tempreg], %[bods_bodse]" "\n\t" \
                       "out %[mcucr], %[tempreg]" "\n\t" \
                       "andi %[tempreg], %[not_bodse]" "\n\t" \
                       "out %[mcucr], %[tempreg]" \
                       : [tempreg] "=&d" (tempreg) \
                       : [mcucr] "I" _SFR_IO_ADDR(MCUCR), \
                         [bods_bodse] "i" (_BV(BODS) | _BV(BODSE)), \
                         [not_bodse] "i" (~_BV(BODSE))); \
}

static DateTime interruptTime;

void powMan_ei(){		//Enable Interrupt 初回
	/*Initialize INT0 pin for accepting interrupts */
	PORTD |= 0x04; 
	DDRD &=~ 0x04;
	pinMode(4, INPUT);                      	// extern power

	attachInterrupt(0, INT0_ISR, LOW);		// Only LOW level interrupt can wake up from PWR_DOWN
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	DateTime  now = RTC.now();			// RTCを主スケッチで定義しておくこと
	interruptTime = DateTime( now.get() + SLEEP_DURATION );
}

void powMan_sleep(){
	//Enable Interrupt 
	DateTime  now = RTC.now();
	while( interruptTime.get() < (now.get() + 10 ) ){
		interruptTime = DateTime(interruptTime.get() + SLEEP_DURATION);
	}
	RTC.clearINTStatus(); //This function call is  a must to bring /INT pin HIGH after an interrupt.
	RTC.enableInterrupts(interruptTime.hour(),interruptTime.minute(),interruptTime.second());    // set the interrupt at (h,m,s)
	attachInterrupt(0, INT0_ISR, LOW);  //Enable INT0 interrupt (as ISR disables interrupt). This strategy is required to handle LEVEL triggered interrupt
	//Power Down routines
	cli(); 
	sleep_enable();			// Set sleep enable bit
	sleep_bod_disable();	// Disable brown out detection during sleep. Saves more power
	sei();
	delay(10);
	//Shut down all peripherals like ADC before sleep. Refer Atmega328 manual
	power_all_disable();	//This shuts down ADC, TWI, SPI, Timers and USART
	sleep_cpu();			// Sleep the CPU as per the mode set earlier(power down)  
	sleep_disable();		// Wakes up sleep and clears enable bit. Before this ISR would have executed
	power_all_enable();		//This shuts enables ADC, TWI, SPI, Timers and USART
	delay(10);				//This delay is required to allow CPU to stabilize
}

//Interrupt service routine for external interrupt on INT0 pin conntected to DS3231 /INT
void INT0_ISR(){
	//Keep this as short as possible. Possibly avoid using function calls
	detachInterrupt(0); 
	interruptTime = DateTime(interruptTime.get() + SLEEP_DURATION);  //decide the time for next interrupt, configure next interrupt  
}
